@@ -192,7 +192,9 @@ int timeline_request(snac *snac, char **id, xs_str **wrk, int level)
                         type = "(null)";
                 }
 
-                if (strcmp(type, "Note") == 0 || strcmp(type, "Page") == 0) {
+                if (strcmp(type, "Note") == 0 ||
+                    strcmp(type, "Page") == 0 ||
+                    strcmp(type, "Article") == 0) {
                     const char *actor = xs_dict_get(object, "attributedTo");
 
                     /* request (and drop) the actor for this entry */
@@ -1122,6 +1124,10 @@ xs_dict *msg_note(snac *snac, const xs_str *content, const xs_val *rcpts,
             if ((v = xs_dict_get(p_msg, "context")))
                 ctxt = xs_dup(v);
 
+            /* propagate the conversation field, if there is one */
+            if ((v = xs_dict_get(p_msg, "conversation")))
+                msg = xs_dict_append(msg, "conversation", v);
+
             /* if this message is public, ours will also be */
             if (!priv && is_msg_public(p_msg) && xs_list_in(to, public_address) == -1)
                 to = xs_list_append(to, public_address);
@@ -2059,6 +2065,10 @@ int activitypub_get_handler(const xs_dict *req, const char *q_path,
         xs *id = xs_fmt("%s/%s", snac.actor, p_path);
 
         status = object_get(id, &msg);
+
+        /* don't return non-public objects */
+        if (valid_status(status) && !is_msg_public(msg))
+            status = 404;
     }
     else
         status = 404;
