@@ -1538,21 +1538,10 @@ int limited(snac *user, const char *id, int cmd)
 
 /** static data **/
 
-xs_str *_static_fn(snac *snac, const char *id)
-/* gets the filename for a static file */
+static int _load_raw_file(const char *fn, xs_val **data, int *size,
+                        const char *inm, xs_str **etag)
+/* loads a cached file */
 {
-    if (strchr(id, '/'))
-        return NULL;
-    else
-        return xs_fmt("%s/static/%s", snac->basedir, id);
-}
-
-
-int static_get(snac *snac, const char *id, xs_val **data, int *size,
-                const char *inm, xs_str **etag)
-/* returns static content */
-{
-    xs *fn = _static_fn(snac, id);
     int status = 404;
 
     if (fn) {
@@ -1584,7 +1573,7 @@ int static_get(snac *snac, const char *id, xs_val **data, int *size,
             if (etag != NULL)
                 *etag = xs_dup(e);
 
-            srv_debug(1, xs_fmt("static_get(): %s %d", id, status));
+            srv_debug(1, xs_fmt("_load_raw_file(): %s %d", fn, status));
         }
     }
 
@@ -1592,6 +1581,26 @@ int static_get(snac *snac, const char *id, xs_val **data, int *size,
 }
 
 
+xs_str *_static_fn(snac *snac, const char *id)
+/* gets the filename for a static file */
+{
+    if (strchr(id, '/'))
+        return NULL;
+    else
+        return xs_fmt("%s/static/%s", snac->basedir, id);
+}
+
+
+int static_get(snac *snac, const char *id, xs_val **data, int *size,
+                const char *inm, xs_str **etag)
+/* returns static content */
+{
+    xs *fn = _static_fn(snac, id);
+
+    return _load_raw_file(fn, data, size, inm, etag);
+}
+
+
 void static_put(snac *snac, const char *id, const char *data, int size)
 /* writes status content */
 {
@@ -1668,7 +1677,8 @@ double history_mtime(snac *snac, const char *id)
 }
 
 
-void history_add(snac *snac, const char *id, const char *content, int size)
+void history_add(snac *snac, const char *id, const char *content, int size,
+                    xs_str **etag)
 /* adds something to the history */
 {
     xs *fn = _history_fn(snac, id);
@@ -1677,22 +1687,21 @@ void history_add(snac *snac, const char *id, const char *content, int size)
     if (fn && (f = fopen(fn, "w")) != NULL) {
         fwrite(content, size, 1, f);
         fclose(f);
+
+        if (etag) {
+            double tm = mtime(fn);
+            *etag = xs_fmt("W/\"snac-%.0lf\"", tm);
+        }
     }
 }
 
 
-xs_str *history_get(snac *snac, const char *id)
+int history_get(snac *snac, const char *id, xs_str **content, int *size,
+                const char *inm, xs_str **etag)
 {
-    xs_str *content = NULL;
     xs *fn = _history_fn(snac, id);
-    FILE *f;
 
-    if (fn && (f = fopen(fn, "r")) != NULL) {
-        content = xs_readall(f);
-        fclose(f);
-    }
-
-    return content;
+    return _load_raw_file(fn, content, size, inm, etag);
 }
 
 
@@ -1711,7 +1720,7 @@ xs_list *history_list(snac *snac)
 {
     xs *spec = xs_fmt("%s/history/" "*.html", snac->basedir);
 
-    return xs_glob(spec, 1, 0);
+    return xs_glob(spec, 1, 1);
 }
 
 
