@@ -35,6 +35,8 @@ int usage(void)
     printf("unpin {basedir} {uid} {msg_url}     Unpins a message\n");
     printf("block {basedir} {instance_url}      Blocks a full instance\n");
     printf("unblock {basedir} {instance_url}    Unblocks a full instance\n");
+    printf("limit {basedir} {uid} {actor}       Limits an actor (drops their announces)\n");
+    printf("unlimit {basedir} {uid} {actor}     Unlimits an actor\n");
 
 /*    printf("question {basedir} {uid} 'opts'  Generates a poll (;-separated opts)\n");*/
 
@@ -53,8 +55,6 @@ char *get_argv(int *argi, int argc, char *argv[])
 
 #define GET_ARGV() get_argv(&argi, argc, argv)
 
-d_char *html_timeline(snac *snac, char *list, int local);
-
 int main(int argc, char *argv[])
 {
     char *cmd;
@@ -198,7 +198,7 @@ int main(int argc, char *argv[])
         xs *list = index_list_desc(idx, 0, 256);
         xs *tl   = timeline_top_level(&snac, list);
 
-        xs_json_dump_pp(tl, 4, stdout);
+        xs_json_dump(tl, 4, stdout);
 
         return 0;
     }
@@ -229,7 +229,7 @@ int main(int argc, char *argv[])
             enqueue_message(&snac, msg);
 
             if (dbglevel) {
-                xs_json_dump_pp(msg, 4, stdout);
+                xs_json_dump(msg, 4, stdout);
             }
         }
 
@@ -247,7 +247,7 @@ int main(int argc, char *argv[])
             enqueue_output_by_actor(&snac, msg, actor, 0);
 
             if (dbglevel) {
-                xs_json_dump_pp(msg, 4, stdout);
+                xs_json_dump(msg, 4, stdout);
             }
         }
 
@@ -272,6 +272,34 @@ int main(int argc, char *argv[])
         return 0;
     }
 
+    if (strcmp(cmd, "limit") == 0) { /** **/
+        int ret;
+
+        if (!following_check(&snac, url))
+            snac_log(&snac, xs_fmt("actor %s is not being followed", url));
+        else
+        if ((ret = limit(&snac, url)) == 0)
+            snac_log(&snac, xs_fmt("actor %s is now limited", url));
+        else
+            snac_log(&snac, xs_fmt("error limiting actor %s (%d)", url, ret));
+
+        return 0;
+    }
+
+    if (strcmp(cmd, "unlimit") == 0) { /** **/
+        int ret;
+
+        if (!following_check(&snac, url))
+            snac_log(&snac, xs_fmt("actor %s is not being followed", url));
+        else
+        if ((ret = unlimit(&snac, url)) == 0)
+            snac_log(&snac, xs_fmt("actor %s is no longer limited", url));
+        else
+            snac_log(&snac, xs_fmt("error unlimiting actor %s (%d)", url, ret));
+
+        return 0;
+    }
+
     if (strcmp(cmd, "ping") == 0) { /** **/
         xs *actor_o = NULL;
 
@@ -281,7 +309,7 @@ int main(int argc, char *argv[])
             enqueue_output_by_actor(&snac, msg, url, 0);
 
             if (dbglevel) {
-                xs_json_dump_pp(msg, 4, stdout);
+                xs_json_dump(msg, 4, stdout);
             }
         }
         else {
@@ -320,7 +348,7 @@ int main(int argc, char *argv[])
         xs *c_msg = msg_create(&snac, msg);
 
         if (dbglevel) {
-            xs_json_dump_pp(c_msg, 4, stdout);
+            xs_json_dump(c_msg, 4, stdout);
         }
 
         enqueue_message(&snac, c_msg);
@@ -340,7 +368,7 @@ int main(int argc, char *argv[])
         printf("status: %d\n", status);
 
         if (data != NULL) {
-            xs_json_dump_pp(data, 4, stdout);
+            xs_json_dump(data, 4, stdout);
         }
 
         return 0;
@@ -355,7 +383,7 @@ int main(int argc, char *argv[])
         printf("status: %d\n", status);
 
         if (valid_status(status)) {
-            xs_json_dump_pp(data, 4, stdout);
+            xs_json_dump(data, 4, stdout);
         }
 
         return 0;
@@ -398,7 +426,7 @@ int main(int argc, char *argv[])
         c_msg = msg_create(&snac, msg);
 
         if (dbglevel) {
-            xs_json_dump_pp(c_msg, 4, stdout);
+            xs_json_dump(c_msg, 4, stdout);
         }
 
         enqueue_message(&snac, c_msg);
