@@ -12,6 +12,7 @@
 #include "xs_set.h"
 #include "xs_random.h"
 #include "xs_httpd.h"
+#include "xs_mime.h"
 
 #include "snac.h"
 
@@ -668,10 +669,16 @@ xs_dict *mastoapi_status(snac *snac, const xs_dict *msg)
         const char *name    = xs_dict_get(msg, "name");
         xs *s1 = NULL;
 
-        if (name)
+        if (name && content)
             s1 = xs_fmt("%s<br><br>%s", name, content);
         else
+        if (name)
+            s1 = xs_dup(name);
+        else
+        if (content)
             s1 = xs_dup(content);
+        else
+            s1 = xs_str_new(NULL);
 
         st = xs_dict_append(st, "content", s1);
     }
@@ -702,7 +709,10 @@ xs_dict *mastoapi_status(snac *snac, const xs_dict *msg)
         attr_list = xs_list_append(attr_list, att);
     }
     else
+    if (xs_type(att) == XSTYPE_LIST)
         attr_list = xs_dup(att);
+    else
+        attr_list = xs_list_new();
 
     /* if it has an image, add it as an attachment */
     xs_dict *image = xs_dict_get(msg, "image");
@@ -715,6 +725,22 @@ xs_dict *mastoapi_status(snac *snac, const xs_dict *msg)
         if (xs_is_null(mtype))
             mtype = xs_dict_get(aobj, "type");
 
+        const char *url = xs_dict_get(aobj, "url");
+        if (xs_is_null(url))
+            url = xs_dict_get(aobj, "href");
+        if (xs_is_null(url))
+            continue;
+
+        /* if it's a plain Link, check if it can be "rewritten" */
+        if (xs_list_len(attr_list) < 2 && strcmp(mtype, "Link") == 0) {
+            const char *mt = xs_mime_by_ext(url);
+
+            if (xs_startswith(mt, "image/") ||
+                xs_startswith(mt, "audio/") ||
+                xs_startswith(mt, "video/"))
+                mtype = mt;
+        }
+
         if (!xs_is_null(mtype)) {
             if (xs_startswith(mtype, "image/") || xs_startswith(mtype, "video/") ||
                 strcmp(mtype, "Image") == 0) {
@@ -723,9 +749,9 @@ xs_dict *mastoapi_status(snac *snac, const xs_dict *msg)
 
                 matte = xs_dict_append(matte, "id",          matteid);
                 matte = xs_dict_append(matte, "type",        *mtype == 'v' ? "video" : "image");
-                matte = xs_dict_append(matte, "url",         xs_dict_get(aobj, "url"));
-                matte = xs_dict_append(matte, "preview_url", xs_dict_get(aobj, "url"));
-                matte = xs_dict_append(matte, "remote_url",  xs_dict_get(aobj, "url"));
+                matte = xs_dict_append(matte, "url",         url);
+                matte = xs_dict_append(matte, "preview_url", url);
+                matte = xs_dict_append(matte, "remote_url",  url);
 
                 const char *name = xs_dict_get(aobj, "name");
                 if (xs_is_null(name))
@@ -744,11 +770,25 @@ xs_dict *mastoapi_status(snac *snac, const xs_dict *msg)
         xs *ml  = xs_list_new();
         xs *htl = xs_list_new();
         xs *eml = xs_list_new();
-        xs_list *p = xs_dict_get(msg, "tag");
-        xs_dict *v;
+        xs_list *tag = xs_dict_get(msg, "tag");
         int n = 0;
 
-        while (xs_list_iter(&p, &v)) {
+        xs *tag_list = NULL;
+
+        if (xs_type(tag) == XSTYPE_DICT) {
+            tag_list = xs_list_new();
+            tag_list = xs_list_append(tag_list, tag);
+        }
+        else
+        if (xs_type(tag) == XSTYPE_LIST)
+            tag_list = xs_dup(tag);
+        else
+            tag_list = xs_list_new();
+
+        tag = tag_list;
+        xs_dict *v;
+
+        while (xs_list_iter(&tag, &v)) {
             const char *type = xs_dict_get(v, "type");
 
             if (xs_is_null(type))
@@ -980,7 +1020,7 @@ int mastoapi_get_handler(const xs_dict *req, const char *q_path,
         if (logged_in) {
             xs *acct = xs_dict_new();
 
-            acct = xs_dict_append(acct, "id",           xs_dict_get(snac1.config, "uid"));
+            acct = xs_dict_append(acct, "id",           snac1.md5);
             acct = xs_dict_append(acct, "username",     xs_dict_get(snac1.config, "uid"));
             acct = xs_dict_append(acct, "acct",         xs_dict_get(snac1.config, "uid"));
             acct = xs_dict_append(acct, "display_name", xs_dict_get(snac1.config, "name"));
@@ -1524,11 +1564,32 @@ int mastoapi_get_handler(const xs_dict *req, const char *q_path,
 
             xs *d13 = xs_json_loads("{\"image_matrix_limit\":33177600,"
                         "\"image_size_limit\":16777216,"
-                        "\"supported_mime_types\":[\"image/jpeg\"],"
                         "\"video_frame_rate_limit\":120,"
                         "\"video_matrix_limit\":8294400,"
                         "\"video_size_limit\":103809024}"
             );
+
+            {
+                /* get the supported mime types from the internal list */
+                const char **p = xs_mime_types;
+                xs_set mtypes;
+                xs_set_init(&mtypes);
+
+                while (*p) {
+                    const char *type = p[1];
+
+                    if (xs_startswith(type, "image/") ||
+                        xs_startswith(type, "video/") ||
+                        xs_startswith(type, "audio/"))
+                        xs_set_add(&mtypes, type);
+
+                    p += 2;
+                }
+
+                xs *l = xs_set_result(&mtypes);
+                d13 = xs_dict_append(d13, "supported_mime_types", l);
+            }
+
             cfg = xs_dict_append(cfg, "media_attachments", d13);
 
             xs *d14 = xs_json_loads("{\"max_characters_per_option\":50,"
