@@ -1,7 +1,7 @@
 /* snac - A simple, minimalistic ActivityPub instance */
 /* copyright (c) 2022 - 2023 grunfink et al. / MIT license */
 
-#define VERSION "2.40"
+#define VERSION "2.41"
 
 #define USER_AGENT "snac/" VERSION
 
@@ -149,8 +149,10 @@ void static_put_meta(snac *snac, const char *id, const char *str);
 xs_str *static_get_meta(snac *snac, const char *id);
 
 double history_mtime(snac *snac, const char *id);
-void history_add(snac *snac, const char *id, const char *content, int size);
-xs_str *history_get(snac *snac, const char *id);
+void history_add(snac *snac, const char *id, const char *content, int size,
+                    xs_str **etag);
+int history_get(snac *snac, const char *id, xs_str **content, int *size,
+                const char *inm, xs_str **etag);
 int history_del(snac *snac, const char *id);
 xs_list *history_list(snac *snac);
 
